<?php

namespace RepairPluginPro;

if ( ! defined( 'RP_DP_PLUGIN_PATH' ) ) {
	die;
}

class LongProcess {

    public $saveHandler;

    protected $repeatAfterXSeconds = 3600;

    protected $steps = array();

    protected $step_names = array();

    public $output = '';

    public function __construct( $saveHandler, $repeatAfterXSeconds = 3600 ) {
        
        $this->saveHandler = $saveHandler;

        $this->repeatAfterXSeconds = $repeatAfterXSeconds;

    }

    public function register_process( $step = '', $callback = null ) {

        $this->steps[ $step ] = $callback;

        $this->step_names[] = $step;

    }

    public function register_repeat_process() {

        // register our own process that will delay repeating the process until X seconds have passed

        $saveHandler = $this->saveHandler;

        $repeatAfterXSeconds = $this->repeatAfterXSeconds;

        $this->register_process( 'repeat', function( $page, $that ) use ( $saveHandler, $repeatAfterXSeconds ) {

            if( $page === 1 ) {

                // save the current time
                $saveHandler->save_last_time( time() );

            }
            
            // get current time
            $current_time = time();

            // get last time
            $last_time = $saveHandler->get_last_time();

            // check if X seconds have passed
            if( ($current_time - $last_time) < $repeatAfterXSeconds ) {

                // not enough time has passed, so return 1 to repeat this step
                if( $page === 1 ) {
                    $repeatInValue = ($repeatAfterXSeconds - ($current_time - $last_time));
                    // convert to minutes
                    $repeatInValue = round( $repeatInValue / 60 );
                    $that->output = "The process will repeat in " . $repeatInValue . " minutes";
                }
                return 2;

            }

            $that->output = "X seconds have passed, so the process will continue from next request";
            // X seconds have passed, so return TRUE to continue to the next step
            return TRUE;

        } );

    }

    public function process() {

        $this->register_repeat_process();

        // if no registered steps, return
        if ( empty( $this->steps ) ) {

            return;

        }

        list($step, $page) = $this->saveHandler->get_saved_step();

        if( empty( $step ) ) {

            // get first step
            $step = $this->step_names[0];

        }

        if ( ! empty( $this->steps[ $step ] ) ) {

            // track time
            $start_time = microtime( true );

            // start transaction
            $this->saveHandler->start_transaction();

            $page_or_completed = $this->steps[ $step ]( (int) $page, $this );

            $this->saveHandler->commit();

            $end_time = microtime( true );

            if( !empty( $this->output ) ) {

                $total_time = $end_time - $start_time;

                $this->log_outputs( $step , $this->output, $total_time );

            }

        }

        if ( $page_or_completed === TRUE ) {

            // get key from $step_names that come after $step
            $next_step = array_search( $step, $this->step_names ) + 1;

            // $next_step contains the key of the next step
            // so we can get the name of the next step
            $next_step = $this->step_names[ $next_step ] ?? '';

            // if there is no next step, we are done
            if ( empty( $next_step ) ) {

                // repeat from the beginning
                $next_step = $this->step_names[0];

            }

            // now we can save the next step
            $this->saveHandler->save_step( $next_step, 1 );

        } else {

            // it is a page number, so save it
            $this->saveHandler->save_step( $step, $page_or_completed );

        }

    }

    public function get_output() {

        return $this->output;

    }

    public function calculatePagination( $data = array(), $page = 0, $limit = 0) {

        $data_count = count( $data );

        $total_pages = ceil( count( $data ) / $limit );

        $end_items = $page * $limit;

        if( $end_items > count( $data ) ) {

            $end_items = count( $data );

        }

        $offset = ($page * $limit) - $limit;

        $limited_data = array_slice( $data, $offset, $limit );

        return array( $limited_data, $data_count, $total_pages, $end_items, $offset );

    }

    public function log_outputs( $step = '', $log_message = '', $script_time = '' ) {

        $script_time = round( $script_time, 4 );

        $outputs = $this->saveHandler->get_option('outputs');

        if( empty( $outputs ) ) {

            $outputs = array();

        } else {

            $outputs = json_decode( $outputs, true );

        }

        $outputs[] = '[' . date( 'Y-m-d H:i:s' ) . '] ' . strtoupper($step) . ': ' . $log_message . ' --- ' . $script_time . ' seconds';

        // limit to 100 outputs
        $outputs = array_slice( $outputs, -100 );

        $this->saveHandler->save_or_update( 'outputs', json_encode( $outputs ) );

    }

    public function log_errors( $log_message = '' ) {

        $fileName = 'errors.log';

        $filePath = RP_DP_PLUGIN_PATH . $fileName;

        if( file_exists( $filePath ) ) {

            $fileContent = file_get_contents( $filePath );

        } else {

            $fileContent = '';

        }

        if( empty( $fileContent ) ) {

            $errors = array();

        } else {

            $errors = unserialize( $fileContent );

        }

        $errors[] = '[' . date( 'Y-m-d H:i:s' ) . '] ' . $log_message;

        // limit to 100 errors
        $errors = array_slice( $errors, -100 );

        // save
        file_put_contents( $filePath, serialize( $errors ) );

    }

    public function has_lock_file() {

        $file = dirname( __FILE__ ) . '/lock.txt';

        if( file_exists( $file ) ) {

            $content = file_get_contents( $file );

            if( empty( $content ) ) {

                $this->delete_lock_file();

                return false;

            }

            $created_time = (int) $content;

            if( $created_time > ( time() - 600 ) ) {

                // lock file was created less than 10 minutes ago
                return true;

            } else {

                // looks like the lock file is old, so delete it
                $this->delete_lock_file();

            }

            return false;

        }

        return false;

    }

    public function create_lock_file() {

        $file = dirname( __FILE__ ) . '/lock.txt';

        if( file_exists( $file ) ) {

            return false;

        }

        file_put_contents( $file, time() );

    }

    public function delete_lock_file() {

        @unlink( dirname( __FILE__ ) . '/lock.txt' );

    }

}

// The above class can be used when you have a long process that you want to run in the background.
// You can register steps and callbacks for each step.
// The process() method will run the next step in the process.
// The saveHandler is used to save the current step and page number.
// The saveHandler can be a file, database, or anything else you want.
// The saveHandler can also be used to retrieve the current step and page number.

// Example usage:

// $saveHandler = new LongProcessSaveHandler();

// $longProcess = new LongProcess( $saveHandler, 5 );

// $longProcess->register_process( 'step1', function( $page, $that ) {

//     $xml_url = 'https://files.channable.com/CWJpBbHSuiIVuVl8JIrOZQ==.xml';

//     $content = file_get_contents( $xml_url );

//     $xml = simplexml_load_string( $content );

//     // new array
//     $all_data = array();

//     // loop through all $xml->item elements
//     foreach( $xml->item as $item ) {

//         $item = (array) $item;

//         $all_data[] = $item;

//     }

//     // save data in channable.txt
//     file_put_contents( dirname( __FILE__ ) . '/channable.txt', serialize( $all_data ) );

    // $that->output = "Downloaded channable datafeed";

    // return TRUE;

// } );

// $longProcess->register_process( 'step2', function( $page, $that ) {

//     $content = file_get_contents( dirname( __FILE__ ) . '/channable.txt' );

//     $content = unserialize( $content );

//     $total_pages = ceil( count( $content ) / 100 );

//     $end_items = $page * 100;

//     if( $end_items > count( $content ) ) {

//         $end_items = count( $content );

//     }

//     $that->output = "Processing & saving content from channable: " .(($page * 100) - 100) . " to " . ($end_items) . " of " . count( $content ) . " items";

//     if( $page + 1 > $total_pages ) {

//         // we are done
//         return TRUE;

//     } else {

//         // we are not done, so return the next page number
//         return $page + 1;

//     }

// } );

// $longProcess->register_process( 'step3', function( $page, $that ) {

//     $content = file_get_contents( dirname( __FILE__ ) . '/channable.txt' );

//     $content = unserialize( $content );

//     $total_pages = ceil( count( $content ) / 100 );

//     $end_items = $page * 100;

//     if( $end_items > count( $content ) ) {

//         $end_items = count( $content );

//     }

//     $that->output = "Try to create automatic linking: " .(($page * 100) - 100) . " to " . ($end_items) . " of " . count( $content ) . " items";

//     if( $page + 1 > $total_pages ) {

//         // we are done
//         return TRUE;

//     } else {

//         // we are not done, so return the next page number
//         return $page + 1;

//     }

// } );

// $longProcess->process();

// echo $longProcess->get_output();
